@@ -249,10 +249,11 @@ def delete_cell():
 @click.command()
 @click.option('--sync', multiple=True, default=None, help="Optional stream IDs for sync.")
 @click.option('--stream', multiple=True, default=None, help="Optional stream ID for stream.")
-def init_node(sync, stream):
-    asyncio.run(async_init_node(sync, stream))
+@click.option('--app', is_flag=True, help="Generate a Node with app template")
+def init_node(sync, stream, app):
+    asyncio.run(async_init_node(sync, stream, app))
 
-async def async_init_node(sync, stream):
+async def async_init_node(sync, stream, app):
     credentials_folder_path = Path.home() / ".neuronum"
     env_path = credentials_folder_path / ".env"
 
@@ -304,14 +305,12 @@ async def async_init_node(sync, stream):
     tx = await cell.list_tx()
     ctx = await cell.list_ctx()
     stx = await cell.list_stx()
-    contracts = await cell.list_contracts()
     nodes = await cell.list_nodes()
 
     await asyncio.to_thread((project_path / "cells.json").write_text, json.dumps(cells, indent=4))
     await asyncio.to_thread((project_path / "transmitters.json").write_text, json.dumps(tx, indent=4))
     await asyncio.to_thread((project_path / "circuits.json").write_text, json.dumps(ctx, indent=4))
     await asyncio.to_thread((project_path / "streams.json").write_text, json.dumps(stx, indent=4))
-    await asyncio.to_thread((project_path / "contracts.json").write_text, json.dumps(contracts, indent=4))
     await asyncio.to_thread((project_path / "nodes.json").write_text, json.dumps(nodes, indent=4))
 
     env_path = project_path / ".env"
@@ -321,8 +320,49 @@ async def async_init_node(sync, stream):
     await asyncio.to_thread(gitignore_path.write_text, ".env\n")
 
     nodemd_path = project_path / "NODE.md"
-    await asyncio.to_thread(nodemd_path.write_text, "## Use this NODE.md file to add instructions on how to interact with your node\n")
+    await asyncio.to_thread(nodemd_path.write_text, """### NODE.md: How to interact with this Node
 
+```json
+{
+    "info": {
+        "use_case": "This Node...",
+        "github": "https://github.com/user"
+    },
+    "gateways": [
+        {
+            "type": "stream",
+            "id": "id::stx",
+            "link": "https://neuronum.net/stream/id::stx"
+        },
+        {
+            "type": "transmitter",
+            "id": "id::tx",
+            "link": "https://neuronum.net/tx/id::tx", 
+        }
+        {
+            "type": "circuit",
+            "id": "id::ctx",
+            "link": "https://neuronum.net/circuit/id::ctx", 
+        }
+    ],
+    "initialization": {
+        "command": "neuronum init-node --sync id::stx"
+    },
+    "requirements": [
+        {
+            "name": "Python",
+            "version": ">= 3.8",
+            "link": "https://www.python.org/downloads/"
+        },
+        {
+            "name": "neuronum",
+            "version": ">= 5.1.0",
+            "link": "https://pypi.org/project/neuronum/"
+        }
+    ]
+}
+```"""
+)
 
     stx = sync[0] if sync else (stream[0] if stream else host.replace("::cell", "::stx"))
 
@@ -399,7 +439,7 @@ async def main():
 asyncio.run(main())
 """)
     
-    if not sync and not stream:
+    if not sync and not stream and not app:
         sync_path = project_path / f"sync_{stx.replace('::stx', '')}.py"
         sync_path.write_text(f"""\
 import asyncio
@@ -421,8 +461,7 @@ cell = neuronum.Cell(
 )
 
 async def main():
-    STX = "{stx}"
-    async for operation in cell.sync(STX):
+    async for operation in cell.sync():
         message = operation.get("data").get("message")
         print(message)
 
@@ -450,20 +489,20 @@ cell = neuronum.Cell(
 )
 
 async def main():
-    STX = "{stx}"
     label = "Welcome to Neuronum"
     
     while True:
         data = {{
             "message": "Hello, Neuronum!"
         }}
-        await cell.stream(label, data, STX)
+        await cell.stream(label, data)
 
 asyncio.run(main())
 """)
         
-    scan_path = project_path / f"scan.py"
-    scan_path.write_text(f"""\
+    if app:
+        app_path = project_path / "app.py"
+        app_path.write_text(f"""\
 import asyncio
 import neuronum
 import os
@@ -483,36 +522,62 @@ cell = neuronum.Cell(
 )
 
 async def main():
-    async for cp in cell.scan():
-        print(cp)
+    STX = "id::stx"                        
+    async for operation in cell.sync(STX):       
+        txID = operation.get("txID")
+        client = operation.get("operator")                    
+                            
+        if txID == "id::tx":             
+            data = {{
+                "response": "TX activated!"
+            }}
+            await cell.tx_response(txID, client, data)
+
+        if txID == "id::tx":
+            data = {{
+                "response": "TX activated!"
+            }}
+            await cell.tx_response(txID, client, data)
+
+        if txID == "id::tx":
+            data = {{
+                "response": "TX activated!"
+            }}
+            await cell.tx_response(txID, client, data)
 
 asyncio.run(main())
 """)
-
+        
     click.echo(f"Neuronum Node '{nodeID}' initialized!")
 
 
 @click.command()
-def start_node():
-    scan_type = questionary.select(
-        "Scan for Neuronum Cells and Nodes (Ensure Bluetooth is enabled)",
-        choices=["On", "Off"]
-    ).ask()
-
+@click.option('--d', is_flag=True, help="Start node in detached mode")
+def start_node(d):
     click.echo("Starting Node...")
 
     project_path = Path.cwd()
-    script_files = glob.glob("sync_*.py") + glob.glob("stream_*.py")
-
-    if scan_type == "On":
-        script_files += glob.glob("scan.py")
+    script_files = glob.glob("sync_*.py") + glob.glob("stream_*.py") + glob.glob("app.py")
 
     processes = []
+    system_name = platform.system()
 
     for script in script_files:
         script_path = project_path / script
         if script_path.exists():
-            process = subprocess.Popen(["python", str(script_path)], start_new_session=True)
+            python_cmd = "pythonw" if system_name == "Windows" else "python"
+
+            if d:
+                process = subprocess.Popen(
+                    ["nohup", python_cmd, str(script_path), "&"] if system_name != "Windows"
+                    else [python_cmd, str(script_path)],
+                    stdout=subprocess.DEVNULL,
+                    stderr=subprocess.DEVNULL,
+                    start_new_session=True
+                )
+            else:
+                process = subprocess.Popen(["python", str(script_path)], start_new_session=True)
+
             processes.append(process.pid)
 
     if not processes:
@@ -525,7 +590,6 @@ def start_node():
     click.echo("Node started successfully!")
 
 
-
 @click.command()
 def stop_node():
     asyncio.run(async_stop_node())
@@ -560,10 +624,15 @@ async def async_stop_node():
 
 
 @click.command()
-def register_node():
-    asyncio.run(async_register_node())
+def connect_node():
+    node_type = questionary.select(
+        "Choose Node type:",
+        choices=["public", "private"]
+    ).ask()
+    descr = click.prompt("Node description (max. 25 characters)")
+    asyncio.run(async_connect_node(descr, node_type))
 
-async def async_register_node():
+async def async_connect_node(descr, node_type):
     env_data = {}
     try:
         with open(".env", "r") as f:
@@ -583,13 +652,6 @@ async def async_register_node():
     except Exception as e:
         print(f"Error reading .env file: {e}")
         return
-    
-    node_type = questionary.select(
-        "Choose Node type:",
-        choices=["public", "private"]
-    ).ask()
-
-    descr = click.prompt("Node description (max. 25 characters)")
 
     try:
         with open("NODE.md", "r") as f: 
@@ -602,7 +664,7 @@ async def async_register_node():
         print(f"Error reading NODE.md file: {e}")
         return
 
-    url = f"https://{network}/api/register_node/{node_type}"
+    url = f"https://{network}/api/connect_node/{node_type}"
 
     node = {
         "nodeID": nodeID,
@@ -623,8 +685,14 @@ async def async_register_node():
         except aiohttp.ClientError as e:
             click.echo(f"Error sending request: {e}")
             return
-
-    click.echo(f"Neuronum Node '{nodeID}' registered! Visit: {node_url}")
+        
+    if nodeID == "Node does not exist":
+        click.echo(f"Neuronum Node not found! Make sure you initialized your Node correctly")
+    else:
+        if node_type == "public":
+            click.echo(f"Public Neuronum Node '{nodeID}' connected! Visit: {node_url}")
+        else:   
+            click.echo(f"Private Neuronum Node '{nodeID}' connected!")
 
 
 @click.command()
@@ -695,19 +763,64 @@ async def async_update_node():
     tx = await cell.list_tx()
     ctx = await cell.list_ctx()
     stx = await cell.list_stx()
-    contracts = await cell.list_contracts()
     nodes = await cell.list_nodes()
 
     await asyncio.to_thread(Path("cells.json").write_text, json.dumps(cells, indent=4))
     await asyncio.to_thread(Path("transmitters.json").write_text, json.dumps(tx, indent=4))
     await asyncio.to_thread(Path("circuits.json").write_text, json.dumps(ctx, indent=4))
     await asyncio.to_thread(Path("streams.json").write_text, json.dumps(stx, indent=4))
-    await asyncio.to_thread(Path("contracts.json").write_text, json.dumps(contracts, indent=4))
     await asyncio.to_thread(Path("nodes.json").write_text, json.dumps(nodes, indent=4))
 
     click.echo(f"Neuronum Node '{nodeID}' updated! Visit: {node_url}")
 
 
+@click.command()
+def disconnect_node():
+    asyncio.run(async_disconnect_node())
+
+async def async_disconnect_node():
+    env_data = {}
+
+    try:
+        with open(".env", "r") as f:
+            for line in f:
+                key, value = line.strip().split("=")
+                env_data[key] = value
+
+        nodeID = env_data.get("NODE", "")
+        host = env_data.get("HOST", "")
+        password = env_data.get("PASSWORD", "")
+        network = env_data.get("NETWORK", "")
+        synapse = env_data.get("SYNAPSE", "")
+
+    except FileNotFoundError:
+        click.echo("Error: .env with credentials not found")
+        return
+    except Exception as e:
+        click.echo(f"Error reading .env file: {e}")
+        return
+
+    url = f"https://{network}/api/disconnect_node"
+    node_payload = {
+        "nodeID": nodeID,
+        "host": host,
+        "password": password,
+        "synapse": synapse
+    }
+
+    async with aiohttp.ClientSession() as session:
+        try:
+            async with session.post(url, json=node_payload) as response:
+                response.raise_for_status()
+                data = await response.json()
+                nodeID = data["nodeID"]
+        except aiohttp.ClientError as e:
+            click.echo(f"Error sending request: {e}")
+            return
+
+    click.echo(f"Neuronum Node '{nodeID}' disconnected!")
+
+
 @click.command()
 def delete_node():
     asyncio.run(async_delete_node())
@@ -755,15 +868,6 @@ async def async_delete_node():
     click.echo(f"Neuronum Node '{nodeID}' deleted!")
 
 
-@click.command() 
-def call_cellai(): 
-    try: 
-        from cellai import cellai 
-        cellai.main() 
-    except ImportError: 
-        click.echo("Cellai not found. Please check the necessary dependencies.")
-
-
 cli.add_command(create_cell)
 cli.add_command(connect_cell)
 cli.add_command(view_cell)
@@ -772,10 +876,10 @@ cli.add_command(delete_cell)
 cli.add_command(init_node)
 cli.add_command(start_node)
 cli.add_command(stop_node)
-cli.add_command(register_node)
+cli.add_command(connect_node)
 cli.add_command(update_node)
+cli.add_command(disconnect_node)
 cli.add_command(delete_node)
-cli.add_command(call_cellai)
 
 
 if __name__ == "__main__":
