@@ -4,7 +4,6 @@ import ssl
 import websockets
 import json
 import asyncio
-from bleak import BleakScanner
 
 class Cell:
     def __init__(self, host: str, password: str, network: str, synapse: str):
@@ -14,6 +13,7 @@ class Cell:
         self.synapse = synapse
         self.queue = asyncio.Queue()
 
+
     def to_dict(self) -> dict:
         return {
             "host": self.host,
@@ -21,8 +21,98 @@ class Cell:
             "synapse": self.synapse
         }
 
+
     def __repr__(self) -> str:
         return f"Cell(host={self.host}, password={self.password}, network={self.network}, synapse={self.synapse})"
+    
+
+    async def stream(self, label: str, data: dict, stx: Optional[str] = None, retry_delay: int = 3):
+        context = ssl.create_default_context()
+        context.check_hostname = True
+        context.verify_mode = ssl.CERT_REQUIRED
+
+        while True:
+            try:
+                reader, writer = await asyncio.open_connection(self.network, 55555, ssl=context, server_hostname=self.network)
+
+                credentials = f"{self.host}\n{self.password}\n{self.synapse}\n{stx}\n"
+                writer.write(credentials.encode("utf-8"))
+                await writer.drain()
+
+                response = await reader.read(1024)
+                response_text = response.decode("utf-8").strip()
+
+                if "Authentication successful" not in response_text:
+                    print("Authentication failed, retrying...")
+                    writer.close()
+                    await writer.wait_closed()
+                    await asyncio.sleep(retry_delay)
+                    continue
+
+                stream_payload = {
+                    "label": label,
+                    "data": data,
+                }
+
+                writer.write(json.dumps(stream_payload).encode("utf-8"))
+                await writer.drain()
+
+                response = await reader.read(1024)
+                response_text = response.decode("utf-8").strip()
+
+                if response_text == "Sent":
+                    print(f"Success: {response_text} - {stream_payload}")
+                    break
+                else:
+                    print(f"Error sending: {stream_payload}")
+
+            except (ssl.SSLError, ConnectionError) as e:
+                print(f"Connection error: {e}, retrying...")
+                await asyncio.sleep(retry_delay)
+
+            except Exception as e:
+                print(f"Unexpected error: {e}, retrying...")
+                await asyncio.sleep(retry_delay)
+
+            finally:
+                if 'writer' in locals():
+                    writer.close()
+                    await writer.wait_closed()
+
+
+    async def sync(self, stx: Optional[str] = None) -> AsyncGenerator[str, None]:
+        full_url = f"wss://{self.network}/sync/{stx}"
+        
+        auth_payload = {
+            "host": self.host,
+            "password": self.password,
+            "synapse": self.synapse,
+        }
+
+        try:
+            async with websockets.connect(full_url) as ws:
+                await ws.send(json.dumps(auth_payload))
+                print("Listening to Stream...")
+
+                try:
+                    while True:
+                        try:
+                            raw_operation = await ws.recv()
+                            operation = json.loads(raw_operation)
+                            yield operation
+
+                        except asyncio.TimeoutError:
+                            print("No initial data received. Continuing to listen...")
+                            continue
+
+                except asyncio.CancelledError:
+                    print("Connection closed.")
+
+        except websockets.exceptions.WebSocketException as e:
+            print(f"WebSocket error occurred: {e}")
+
+        except Exception as e:
+            print(f"An unexpected error occurred: {e}")
            
 
     async def create_tx(self, descr: str, key_values: dict, stx: str, label: str, partners: list):
@@ -73,9 +163,7 @@ class Cell:
     
 
     async def activate_tx(self, txID: str, data: dict):
-        """Asynchronously sends TX activation request to the API."""
         url = f"https://{self.network}/api/activate_tx/{txID}"
-
         TX = {
             "data": data,
             "cell": self.to_dict()
@@ -86,8 +174,38 @@ class Cell:
                 async with session.post(url, json=TX) as response:
                     response.raise_for_status()
                     data = await response.json()
-                    print(f"Response from Neuronum: {data}")
-                    return data
+                    if data["success"] == "activated":
+                        async for operation in self.sync():
+                            label = operation.get("label")
+                            if label == "tx_response":
+                                operation_txID = operation.get("txID")
+                                if operation_txID == txID:
+                                    return operation.get("data")
+                    else:
+                        print(data["success"], data["message"])
+
+            except aiohttp.ClientError as e:
+                print(f"Error sending request: {e}")
+            except Exception as e:
+                print(f"Unexpected error: {e}")
+
+
+    async def tx_response(self, txID: str, client: str, data: dict):
+        url = f"https://{self.network}/api/tx_response/{txID}"
+
+        tx_response = {
+            "client": client,
+            "data": data,
+            "cell": self.to_dict()
+        }
+
+        async with aiohttp.ClientSession() as session:
+            try:
+                for _ in range(2):
+                    async with session.post(url, json=tx_response) as response:
+                        response.raise_for_status()
+                        data = await response.json()
+                print(data["message"])
 
             except aiohttp.ClientError as e:
                 print(f"Error sending request: {e}")
@@ -370,254 +488,4 @@ class Cell:
                 print(f"Unexpected error: {e}")
 
 
-    async def stream(self, label: str, data: dict, stx: Optional[str] = None):
-        context = ssl.create_default_context()
-        context.check_hostname = True
-        context.verify_mode = ssl.CERT_REQUIRED
-
-        try:
-            reader, writer = await asyncio.open_connection(self.network, 55555, ssl=context, server_hostname=self.network)
-
-            credentials = f"{self.host}\n{self.password}\n{self.synapse}\n{stx}\n"
-            writer.write(credentials.encode("utf-8"))
-            await writer.drain()
-
-            response = await reader.read(1024)
-            response_text = response.decode("utf-8")
-
-            if "Authentication successful" not in response_text:
-                print("Authentication failed")
-                writer.close()
-                await writer.wait_closed()
-                return
-
-            stream_payload = {
-                "label": label,
-                "data": data,
-            }
-
-            writer.write(json.dumps(stream_payload).encode("utf-8"))
-            await writer.drain()
-            print(f"Sent: {stream_payload}")
-
-        except ssl.SSLError as e:
-            print(f"SSL error occurred: {e}")
-
-        except Exception as e:
-            print(f"An unexpected error occurred: {e}")
-
-        finally:
-            writer.close()
-            await writer.wait_closed()
-
-
-
-    async def sync(self, stx: Optional[str] = None) -> AsyncGenerator[str, None]:
-        full_url = f"wss://{self.network}/sync/{stx}"
-        
-        auth_payload = {
-            "host": self.host,
-            "password": self.password,
-            "synapse": self.synapse,
-        }
-
-        try:
-            async with websockets.connect(full_url) as ws:
-                await ws.send(json.dumps(auth_payload))
-                print("Listening to Stream...")
-
-                try:
-                    while True:
-                        try:
-                            raw_operation = await ws.recv()
-                            operation = json.loads(raw_operation)
-                            yield operation
-
-                        except asyncio.TimeoutError:
-                            print("No initial data received. Continuing to listen...")
-                            continue
-
-                except asyncio.CancelledError:
-                    print("Connection closed.")
-
-        except websockets.exceptions.WebSocketException as e:
-            print(f"WebSocket error occurred: {e}")
-
-        except Exception as e:
-            print(f"An unexpected error occurred: {e}")
-
-
-    async def sign_contract(self, contractID: str):
-        full_url = f"https://{self.network}/api/sign_contract"
-
-        sign_contract_payload = {
-            "contractID": contractID,
-            "cell": self.to_dict()
-        }
-
-        async with aiohttp.ClientSession() as session:
-            try:
-                async with session.post(full_url, json=sign_contract_payload) as response:
-                    response.raise_for_status()
-                    data = await response.json()
-                    return data.get("token")
-
-            except aiohttp.ClientError as e:
-                print(f"Error sending request: {e}")
-            except Exception as e:
-                print(f"Unexpected error: {e}")
-
-
-    async def validate_token(self, token: str, cp: str, contractID: str):
-        full_url = f"https://{self.network}/api/validate_token"
-
-        validate_payload = {
-            "token": token,
-            "cp": cp,
-            "contractID": contractID,
-            "cell": self.to_dict()
-        }
-
-        async with aiohttp.ClientSession() as session:
-            try:
-                async with session.post(full_url, json=validate_payload) as response:
-                    response.raise_for_status()
-                    data = await response.json()
-                    return data.get("validity")
-
-            except aiohttp.ClientError as e:
-                print(f"Error sending request: {e}")
-            except Exception as e:
-                print(f"Unexpected error: {e}")
-
-
-    async def request_token(self, cp: str, contractID: str):
-        full_url = f"https://{self.network}/api/request_token"
-
-        request_token_payload = {
-            "cp": cp,
-            "contractID": contractID,
-            "cell": self.to_dict()
-        }
-
-        async with aiohttp.ClientSession() as session:
-            try:
-                async with session.post(full_url, json=request_token_payload) as response:
-                    response.raise_for_status()
-                    data = await response.json()
-                    print(f"Response from Neuronum: {data}")
-                    return data
-
-            except aiohttp.ClientError as e:
-                print(f"Error sending request: {e}")
-            except Exception as e:
-                print(f"Unexpected error: {e}")
-
-    
-    async def present_token(self, token: str, cp: str, contractID: str):
-        full_url = f"https://{self.network}/api/present_token"
-
-        present_token_payload = {
-            "token": token,
-            "cp": cp,
-            "contractID": contractID,
-            "cell": self.to_dict()
-        }
-
-        async with aiohttp.ClientSession() as session:
-            try:
-                async with session.post(full_url, json=present_token_payload) as response:
-                    response.raise_for_status()
-                    data = await response.json()
-                    print(f"Response from Neuronum: {data}")
-                    return data
-
-            except aiohttp.ClientError as e:
-                print(f"Error sending request: {e}")
-            except Exception as e:
-                print(f"Unexpected error: {e}")
-
-
-    async def create_contract(self, descr: str, details: dict, partners: list):
-        full_url = f"https://{self.network}/api/create_contract"
-
-        create_contract_payload = {
-            "cell": self.to_dict(),
-            "descr": descr,
-            "details": details,
-            "partners": partners
-        }
-
-        async with aiohttp.ClientSession() as session:
-            try:
-                async with session.post(full_url, json=create_contract_payload) as response:
-                    response.raise_for_status()
-                    data = await response.json()
-                    return data.get("contractID")
-
-            except aiohttp.ClientError as e:
-                print(f"Error sending request: {e}")
-            except Exception as e:
-                print(f"Unexpected error: {e}")
-
-
-    async def delete_contract(self, contractID: str):
-        full_url = f"https://{self.network}/api/delete_contract"
-
-        request_payload = {
-            "cell": self.to_dict(),
-            "contractID": contractID
-        }
-
-        async with aiohttp.ClientSession() as session:
-            try:
-                async with session.post(full_url, json=request_payload) as response:
-                    response.raise_for_status()
-                    data = await response.json()
-                    print(f"Response from Neuronum: {data}")
-                    return data
-
-            except aiohttp.ClientError as e:
-                print(f"Error sending request: {e}")
-            except Exception as e:
-                print(f"Unexpected error: {e}")
-
-
-    async def list_contracts(self):
-        full_url = f"https://{self.network}/api/list_contracts"
-
-        list_contracts_payload = {
-            "cell": self.to_dict()
-        }
-
-        async with aiohttp.ClientSession() as session:
-            try:
-                async with session.get(full_url, json=list_contracts_payload) as response:
-                    response.raise_for_status()
-                    data = await response.json()
-                    return data.get("Contracts", [])
-
-            except aiohttp.ClientError as e:
-                print(f"Error sending request: {e}")
-            except Exception as e:
-                print(f"Unexpected error: {e}")
-
-        
-    def device_found(self, device, advertisement_data):
-        if device.name and (device.name.endswith("::cell") or device.name.endswith("::node")):
-            asyncio.create_task(self.queue.put(f"{device.name} - {device.address}"))
-
-    async def scan(self):
-        print("Scanning for Neuronum Cells & Nodes")
-
-        scanner = BleakScanner(self.device_found)
-        await scanner.start()
-
-        try:
-            while True:
-                yield await self.queue.get()
-        except asyncio.CancelledError:
-            await scanner.stop()
-
-
 __all__ = ['Cell']
